import time
import visa
import socket
import numpy as np

# IP address SMBV100B
ip_address = '10.215.0.219'
rm = visa.ResourceManager()
smw = rm.open_resource('TCPIP::' + ip_address + '::INSTR', write_termination=None)
smw.timeout = 10000

# Reset SMBV100A
smw.query('*RST; *OPC?')

# Configure Hardware in the Loop mode:
# 1. Set operation mode to 'Navigation'
smw.query(':SOUR1:BB:GNSS:TMOD NAV; *OPC?')

# 2. Set Postion to Remote control
smw.query(':SOUR1:BB:GNSS:REC:V1:POS HIL; *OPC?')

# 3. Enter system latency and interface type
smw.query(':SOUR1:BB:GNSS:REC:V1:HIL:SLAT 0.12; *OPC?')
smw.query(':SOUR1:BB:GNSS:REC:V1:HIL:ITYP UDP; *OPC?')

# 4. Set initial position
smw.query(':SOUR1:BB:GNSS:REC:V1:LOC:COOR:DEC:WGS 0.0, 0.0, 10.0; *OPC?')

# 6. Switch on the GNSS simulation and the RF output.
smw.query(':SOUR1:BB:GNSS:STAT ON; *OPC?')
smw.query('OUTP:STAT 1; *OPC?')

sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
address = (ip_address, 7755)

#smw.write('&GTL')
time.sleep(0)

reserveByte12 = 0   #reserved bytes 1+2
reserveByte34 = 0   #reserved bytes 3+4
ElapsedTime = 41    #41 seconds
X = 6378147         #6378147 meters
Y = 0               #0 meters
Z = 0               #0 meters
XDot = 0            #0 meters/second
YDot = 0            #10 meters/second
ZDot = 0            #0 meters/second
XDotDot = 0         #0 meters/second^2
YDotDot = 0         #0 meters/second^2
ZDotDot = 0         #0 meters/second^2
XDotDotDot = 0      #0 meters/second^3
YDotDotDot = 0      #0 meters/second^3
ZDotDotDot = 0      #0 meters/second^3
Roll = 0            #0 rad
Pitch = 0           #0 rad
Yaw = 0             #0 rad
RollDot = 0         #0 rad/second
PitchDot = 0        #0 rad/second
YawDot = 0          #0 rad/second
RollDotDot = 0      #0 rad/second^2
PitchDotDot = 0     #0 rad/second^2
YawDotDot = 0       #0 rad/second^2
RollDotDotDot = 0   #0 rad/second^3
PitchDotDotDot = 0  #0 rad/second^3
YawDotDotDot = 0    #0 rad/second^3

#UDP position data is assembled

waypoint = np.asarray([reserveByte12, reserveByte34, ElapsedTime, X, Y, Z, XDot, YDot, ZDot,
                       XDotDot, YDotDot, ZDotDot, XDotDotDot, YDotDotDot, ZDotDotDot, Roll, Pitch, Yaw, RollDot,
                       PitchDot, YawDot, RollDotDot, PitchDotDot, YawDotDot, RollDotDotDot, PitchDotDotDot,
                       YawDotDotDot], dtype='double')
HILcommand = waypoint.tobytes()

#send UDP position data
sock.sendto(HILcommand, address)
time.sleep(1)

#Only ElapsedTime and Y-coordinate change
ElapsedTime = 42    #42 seconds
Y = 10              #10 meters
waypoint[2] = ElapsedTime
waypoint[4] = Y
HILcommand = waypoint.tobytes()
sock.sendto(HILcommand, address)
time.sleep(1)

ElapsedTime = 43    #43 seconds
Y = 20              #20 meters
waypoint[2] = ElapsedTime
waypoint[4] = Y
HILcommand = waypoint.tobytes()
sock.sendto(HILcommand, address)
time.sleep(1)

ElapsedTime = 44    #44 seconds
Y = 30              #30 meters
waypoint[2] = ElapsedTime
waypoint[4] = Y
HILcommand = waypoint.tobytes()
sock.sendto(HILcommand, address)
time.sleep(1)

ElapsedTime = 45    #45 seconds
Y = 40              #40 meters
waypoint[2] = ElapsedTime
waypoint[4] = Y
HILcommand = waypoint.tobytes()
sock.sendto(HILcommand, address)
time.sleep(1)

sock.close()
smw.close()

