import time
import visa
import socket
import numpy as np

# IP address SMBV100B
ip_address = '10.215.0.219'
rm = visa.ResourceManager()
smw = rm.open_resource('TCPIP::' + ip_address + '::INSTR', write_termination=None)
smw.timeout = 10000

# Reset SMBV100A
smw.query('*RST; *OPC?')

# Configure Hardware in the Loop mode:
# 1. Set operation mode to 'Navigation'
smw.query(':SOUR1:BB:GNSS:TMOD NAV; *OPC?')

# 2. Set Postion to Remote control
smw.query(':SOUR1:BB:GNSS:REC:V1:POS HIL; *OPC?')

# 3. Enter system latency and interface type
smw.query(':SOUR1:BB:GNSS:REC:V1:HIL:SLAT 0.12; *OPC?')
smw.query(':SOUR1:BB:GNSS:REC:V1:HIL:ITYP UDP; *OPC?')

# 4. Set initial position
smw.query(':SOUR1:BB:GNSS:REC:V1:LOC:COOR:DEC:WGS 0.0, 0.0, 10.0; *OPC?')

# 6. Switch on the GNSS simulation and the RF output.
smw.query(':SOUR1:BB:GNSS:STAT ON; *OPC?')
smw.query('OUTP:STAT 1; *OPC?')

sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
address = (ip_address, 7755)

smw.write('&GTL')
time.sleep(0)

reserveByte1 = b'\x00\x00\x00\x00'                  #reserved bits 1
reserveByte2 = b'\x00\x00\x00\x00'                  #reserved bits 2
reserveByte3 = b'\x00\x00\x00\x00'                  #reserved bits 3
reserveByte4 = b'\x00\x00\x00\x00'                  #reserved bits 4
ElapsedTime = b'\x00\x00\x00\x00\x00\x80\x44\x40'   #41 seconds
X = b'\x00\x00\x00\xc0\xa8\x54\x58\x41'             #6378147 meters
Y = b'\x00\x00\x00\x00\x00\x00\x00\x00'             #0 meters
Z = b'\x00\x00\x00\x00\x00\x00\x00\x00'             #0 meters
XDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'          #0 meters/second
YDot = b'\x00\x00\x00\x00\x00\x00\x24\x40'          #0 meters/second
ZDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'          #0 meters/second
XDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'       #0 meters/second^2
YDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'       #0 meters/second^2
ZDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'       #0 meters/second^2
XDotDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'    #0 meters/second^3
YDotDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'    #0 meters/second^3
ZDotDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'    #0 meters/second^3
Roll = b'\x00\x00\x00\x00\x00\x00\x00\x00'          #0 rad
Pitch = b'\x00\x00\x00\x00\x00\x00\x00\x00'         #0 rad
Yaw = b'\x00\x00\x00\x00\x00\x00\x00\x00'           #0 rad
RollDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'       #0 rad/second
PitchDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'      #0 rad/second
YawDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'        #0 rad/second
RollDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'    #0 rad/second^2
PitchDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'   #0 rad/second^2
YawDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'     #0 rad/second^2
RollDotDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00' #0 rad/second^3
PitchDotDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'#0 rad/second^3
YawDotDotDot = b'\x00\x00\x00\x00\x00\x00\x00\x00'  #0 rad/second^3

#UDP position data is assembled
waypoint = reserveByte1 + reserveByte2 + reserveByte3 + reserveByte4 + ElapsedTime + X + Y + Z + XDot + YDot + ZDot + \
           XDotDot + YDotDot + ZDotDot + XDotDotDot + YDotDotDot + ZDotDotDot + Roll + Pitch + Yaw + RollDot + PitchDot \
           + YawDot + RollDotDot + PitchDotDot + YawDotDot + RollDotDotDot + PitchDotDotDot + YawDotDotDot
#send UDP position data
sock.sendto(waypoint, address)
time.sleep(1)

#Only ElapsedTime and Y-coordinate change
ElapsedTime = b'\x00\x00\x00\x00\x00\x00\x45\x40'    #42 seconds
Y = b'\x00\x00\x00\x00\x00\x00\x24\x40'             #10 meters
waypoint = reserveByte1 + reserveByte2 + reserveByte3 + reserveByte4 + ElapsedTime + X + Y + Z + XDot + YDot + ZDot + \
           XDotDot + YDotDot + ZDotDot + XDotDotDot + YDotDotDot + ZDotDotDot + Roll + Pitch + Yaw + RollDot + PitchDot \
           + YawDot + RollDotDot + PitchDotDot + YawDotDot + RollDotDotDot + PitchDotDotDot + YawDotDotDot
sock.sendto(waypoint, address)
time.sleep(1)

ElapsedTime = b'\x00\x00\x00\x00\x00\x80\x45\x40'    #43 seconds
Y = b'\x00\x00\x00\x00\x00\x00\x34\x40'             #20 meters
waypoint = reserveByte1 + reserveByte2 + reserveByte3 + reserveByte4 + ElapsedTime + X + Y + Z + XDot + YDot + ZDot + \
           XDotDot + YDotDot + ZDotDot + XDotDotDot + YDotDotDot + ZDotDotDot + Roll + Pitch + Yaw + RollDot + PitchDot \
           + YawDot + RollDotDot + PitchDotDot + YawDotDot + RollDotDotDot + PitchDotDotDot + YawDotDotDot
sock.sendto(waypoint, address)
time.sleep(1)

ElapsedTime = b'\x00\x00\x00\x00\x00\x00\x46\x40'    #44 seconds
Y = b'\x00\x00\x00\x00\x00\x00\x3e\x40'             #30 meters
waypoint = reserveByte1 + reserveByte2 + reserveByte3 + reserveByte4 + ElapsedTime + X + Y + Z + XDot + YDot + ZDot + \
           XDotDot + YDotDot + ZDotDot + XDotDotDot + YDotDotDot + ZDotDotDot + Roll + Pitch + Yaw + RollDot + PitchDot \
           + YawDot + RollDotDot + PitchDotDot + YawDotDot + RollDotDotDot + PitchDotDotDot + YawDotDotDot
sock.sendto(waypoint, address)
time.sleep(1)

ElapsedTime = b'\x00\x00\x00\x00\x00\x80\x46\x40'    #45 seconds
Y = b'\x00\x00\x00\x00\x00\x00\x44\x40'             #40 meters
waypoint = reserveByte1 + reserveByte2 + reserveByte3 + reserveByte4 + ElapsedTime + X + Y + Z + XDot + YDot + ZDot + \
           XDotDot + YDotDot + ZDotDot + XDotDotDot + YDotDotDot + ZDotDotDot + Roll + Pitch + Yaw + RollDot + PitchDot \
           + YawDot + RollDotDot + PitchDotDot + YawDotDot + RollDotDotDot + PitchDotDotDot + YawDotDotDot
sock.sendto(waypoint, address)
time.sleep(1)

sock.close()
smw.close()

